﻿using System;
using System.Collections.Generic;
using System.IO.Abstractions.TestingHelpers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using TraceLabSDK;
using TraceLabSDK.Component.Config;

namespace ArtifactsParser.Tests
{
    [TestClass]
    public class ArtifactsParserComponentTests
    {
        private IDictionary<string, object> workspace;

        private ArtifactsParserComponent component;

        private MockFileSystem fileSystem;

        [TestInitialize]
        public void TestInitialize()
        {
           this.workspace = new Dictionary<string, object>();

            var logger = new Mock<ComponentLogger>();
            this.component = new ArtifactsParserComponent(logger.Object, this.fileSystem);

            var workspace = new Mock<IWorkspace>();
            workspace.Setup(mock => mock.Load(It.IsAny<string>()))
                .Returns<string>(str => this.workspace[str]);
            workspace.Setup(mock => mock.Store(It.IsAny<string>(), It.IsAny<object>()))
                .Callback<string, object>((str, obj) => this.workspace[str] = obj);
            this.component.Workspace = workspace.Object;

            this.fileSystem = new MockFileSystem();
        }

        [TestMethod]
        public void TestConfigurationIsCustomType()
        {
            Assert.IsInstanceOfType(this.component.Configuration, typeof(ArtifactsParserConfiguration));
        }
    }
}
