﻿using System;
using System.IO.Abstractions.TestingHelpers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TraceLabSDK.Component.Config;

namespace ArtifactsParser.Tests
{
    [TestClass]
    public class ArtifactsParserConfigurationTests
    {
        private ArtifactsParserConfiguration configuration;

        [TestInitialize]
        public void TestInitialize()
        {
            this.configuration = new ArtifactsParserConfiguration(new MockFileSystem());
        }

        [TestMethod]
        public void TestThrowsArgumentNullExceptionWhenArtifactsFileIsNull()
        {
            Assert.ThrowsException<ArgumentNullException>(() => this.configuration.ArtifactsFile = null);
        }

        [TestMethod]
        public void TestThrowsArgumentExceptionWhenArtifactsFileDoesNotExist()
        {
            var path = new FilePath();
            path.Init(@"C:\NotAFile");

            Assert.ThrowsException<ArgumentException>(() => this.configuration.ArtifactsFile = path);
        }

        [TestMethod]
        public void TestThrowsArgumentNullExceptionWhenOutputDirectoryIsNull()
        {
            Assert.ThrowsException<ArgumentNullException>(() => this.configuration.OutputDirectory = null);
        }

        [TestMethod]
        public void TestThrowsArgumentExceptionWhenOutputDirectoryDoesNotExist()
        {
            var path = new DirectoryPath();
            path.Init(@"C:\NotADirectory");

            Assert.ThrowsException<ArgumentException>(() => this.configuration.OutputDirectory = path);
        }
    }
}
