﻿using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace ArtifactsParser.Tests
{
    [TestClass]
    public class ReaderTests
    {
        [TestMethod]
        public void TestPeekReturnsNextCharacter()
        {
            var reader = new Reader("abc");

            Assert.AreEqual('a', reader.Peek());
        }

        [TestMethod]
        public void TestPeekReturnsNullIfNoNextCharacter()
        {
            var reader = new Reader("");

            Assert.IsNull(reader.Peek());
        }

        [TestMethod]
        public void TestGetReturnsNextCharacter()
        {
            var reader = new Reader("abc");

            Assert.AreEqual('a', reader.Get());
        }

        [TestMethod]
        public void TestGetEatsCharacterAfterReturning()
        {
            var reader = new Reader("abc");

            Assert.AreEqual('a', reader.Get());
            Assert.AreEqual('b', reader.Get());
        }

        [TestMethod]
        public void TestGetReturnsNullIfNoNextCharacter()
        {
            var reader = new Reader("abc");

            Assert.AreEqual('a', reader.Get());
            Assert.AreEqual('b', reader.Get());
            Assert.AreEqual('c', reader.Get());
            Assert.IsNull(reader.Get());
        }

        [TestMethod]
        public void TestGetDoesNotIncreaseIndexBeyondOnePastTheEnd()
        {
            var reader = new Reader("abc");

            for (int i = 0; i < 10; ++i)
            {
                reader.Get();
            }
            reader.Rewind(1);

            Assert.AreEqual('c', reader.Peek());
        }

        [TestMethod]
        public void TestSkipWhiteSpaceSkipsAllWhiteSpaceCharacters()
        {
            var reader = new Reader(" \t\n\rabc");

            reader.SkipWhiteSpace();

            Assert.AreEqual('a', reader.Peek());
        }

        [TestMethod]
        public void TestSkipWhiteSpaceDoesntEatNonWhiteSpaceCharacters()
        {
            var reader = new Reader("abc");

            reader.SkipWhiteSpace();

            Assert.AreEqual('a', reader.Peek());
        }

        [TestMethod]
        public void TestRewindSkipsBackwardsByCount()
        {
            var reader = new Reader("abc");
            reader.Get();
            reader.Get();

            reader.Rewind(2);

            Assert.AreEqual('a', reader.Peek());
        }

        [TestMethod]
        public void TestRewindCanNotMakeIndexFallBelowZero()
        {
            var reader = new Reader("abc");

            reader.Rewind(100);

            Assert.AreEqual('a', reader.Peek());
        }
    }
}
