﻿using ArtifactsParser.Sentences;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TraceLabSDK.Types;

namespace ArtifactsParser.Tests.Sentences
{
    [TestClass]
    public class ParsingStrategyTests
    {
        private ParsingStrategy strategy;

        [TestInitialize]
        public void TestInitialize()
        {
            this.strategy = new ParsingStrategy();
        }

        [TestMethod]
        public void TestGetArtifactsSplitsByPeriod()
        {
            var result = strategy.GetArtifacts("asdf.jkl;");

            var expected = new TLArtifactsCollection();
            expected.Add(new TLArtifact("0", "asdf"));
            expected.Add(new TLArtifact("1", "jkl;"));
            CollectionAssert.AreEquivalent(expected, result);
        }

        [TestMethod]
        public void TestGetArtifactsSplitsByQuestionMark()
        {
            var result = strategy.GetArtifacts("asdf?jkl;");

            var expected = new TLArtifactsCollection();
            expected.Add(new TLArtifact("0", "asdf"));
            expected.Add(new TLArtifact("1", "jkl;"));
            CollectionAssert.AreEquivalent(expected, result);
        }

        [TestMethod]
        public void TestGetArtifactsSplitsByExclamationMark()
        {
            var result = strategy.GetArtifacts("asdf!jkl;");

            var expected = new TLArtifactsCollection();
            expected.Add(new TLArtifact("0", "asdf"));
            expected.Add(new TLArtifact("1", "jkl;"));
            CollectionAssert.AreEquivalent(expected, result);
        }

        [TestMethod]
        public void TestGetArtifactsTrimsSentences()
        {
            var result = strategy.GetArtifacts(" asdf . jkl; ");

            var expected = new TLArtifactsCollection();
            expected.Add(new TLArtifact("0", "asdf"));
            expected.Add(new TLArtifact("1", "jkl;"));
            CollectionAssert.AreEquivalent(expected, result);
        }
    }
}
