﻿using System;
using System.IO.Abstractions;
using System.Linq;
using TraceLabSDK;
using TraceLabSDK.Types;

namespace ArtifactsParser
{
    [Component(
        Name = "Policy Parser",
        Description = "Parses artifacts from a file based on a specified strategy.",
        Author = "Satrio Hudoso, Jared Payne",
        Version = "1.0"
    )]
    [IOSpec(IOType = IOSpecType.Output, Name = "Artifacts", DataType = typeof(TLArtifactsCollection))]
    public class ArtifactsParserComponent : BaseComponent
    {
        public ArtifactsParserComponent(ComponentLogger logger) : this(logger, new FileSystem()) { }

        internal ArtifactsParserComponent(ComponentLogger logger, IFileSystem fileSystem) : base(logger)
        {
            this.Configuration = new ArtifactsParserConfiguration(fileSystem);
        }

        public new ArtifactsParserConfiguration Configuration
        {
            get => base.Configuration as ArtifactsParserConfiguration;
            set => base.Configuration = value;
        }

        public override void Compute()
        {
            var path = this.Configuration.ArtifactsFile.Absolute;
            var text = this.Configuration.FileSystem.File.ReadAllText(path);

            var artifacts = this.GetArtifacts(text);
            base.Workspace.Store("Artifacts", artifacts);

            var outputDirectory = this.Configuration.OutputDirectory.Absolute;
            var texts = artifacts.Select(artifact => artifact.Value.Text);
            var paths = artifacts.Select(artifact => string.Format("{0}/S{1}.txt", outputDirectory, artifact.Value.Id));
            paths.Zip(texts, Tuple.Create).ToList().ForEach(tuple => this.Configuration.FileSystem.File.WriteAllText(tuple.Item1, tuple.Item2));
        }

        private TLArtifactsCollection GetArtifacts(string text)
        {
            switch (this.Configuration.ParsingStrategy)
            {
                case ParsingStrategyType.Sentences:
                    return new Sentences.ParsingStrategy().GetArtifacts(text);
                default:
                    throw new ComponentException("Invalid parsing policy given.");
            }
        }
    }
}