﻿using Common;
using System;
using System.ComponentModel;
using System.IO.Abstractions;
using TraceLabSDK.Component.Config;

namespace ArtifactsParser
{
    public class ArtifactsParserConfiguration : FileSystemConfiguration
    {
        private FilePath artifactsFile;

        private DirectoryPath outputDirectory;

        internal ArtifactsParserConfiguration(IFileSystem fileSystem) : base(fileSystem) { }

        [DisplayName("Artifacts location")]
        [Description("The file to parse artifacts from.")]
        public FilePath ArtifactsFile
        {
            get => this.artifactsFile;
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Artifacts file path cannot be null.");
                }
                if (!this.FileSystem.File.Exists(value.Absolute))
                {
                    throw new ArgumentException("Artifacts file path is not a file.");
                }
                this.artifactsFile = value;
            }
        }

        [DisplayName("Output directory")]
        [Description("The directory for writing artifacts as individual files.")]
        public DirectoryPath OutputDirectory
        {
            get => this.outputDirectory;
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Output directory path cannot be null.");
                }
                if (!this.FileSystem.Directory.Exists(value.Absolute))
                {
                    throw new ArgumentException("Output directory path is not a directory.");
                }
                this.outputDirectory = value;
            }
        }

        [Description("The format of the artifacts file.")]
        public ParsingStrategyType ParsingStrategy { get; set; }
    }
}
