﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using TraceLabSDK;
using TraceLabSDK.Types;

namespace BulletsParser
{
    [Component(
        Name = "Bullets Parser",
        Description = "Allows user to specify bulleting format at various levels",
        Author = "Satrio Husodo",
        Version = "1.0",
        ConfigurationType = typeof(BulletsParserConfiguration)
    )]
    [IOSpec(IOSpecType.Output, "outputName", typeof(TLArtifactsCollection))]
    public class BulletsParserComponent : BaseComponent
    {
        public BulletsParserComponent(ComponentLogger log) : base(log)
        {
            this.Configuration = new BulletsParserConfiguration();
        }

        public new BulletsParserConfiguration Configuration
        {
            get => base.Configuration as BulletsParserConfiguration;
            set => base.Configuration = value;
        }

        public override void Compute()
        {
            int i = 0;
            var collection = new TLArtifactsCollection();
            var delimiter = this.Configuration.Delimiter;
            var bulletType1 = this.Configuration.BulletTypeLevel1;
            var bulletType2 = this.Configuration.BulletTypeLevel2;
            var inputDirectory = this.Configuration.Artifacts.Absolute;
            var outputDirectory = this.Configuration.OutputDirectory.Absolute;
            foreach (var item in this.GetItems(File.ReadLines(inputDirectory), bulletType1, bulletType2, delimiter))
            {
                collection.Add(new TLArtifact(Convert.ToString(i), item));
                base.Workspace.Store("outputName", collection);
                var outputFile = outputDirectory + @"\S" + i + ".txt";
                File.WriteAllText(outputFile, item);
                i = i + 1;
            }
        }

        public IEnumerable<string> GetItems(IEnumerable<string> lines, BulletType level1, BulletType level2, string delim)
        {
            var numberSymbols = new string[256];
            for (int i = 0; i < numberSymbols.Length; i++)
            {
                numberSymbols[i] = Convert.ToString(i + 1);
            }
            var letterSymbols = new string[] { "a", "b", "c", "d", "e", "f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z" };
            var romanNumeralSymbols = new string[] { "i", "ii", "iii", "iv", "v", "vi", "vii", "viii", "ix", "x", "xi", "xii", "xiii", "xiv", "xv", "xvi", "xvii", "xviii", "xix", "xx" };
            var originalLevel2Symbols = new string[] { };
            var level1Symbols = new string[] { };
            if (level1 == BulletType.Numbers)
            {
                level1Symbols = numberSymbols;
            }
            else if (level1 == BulletType.Letters)
            {
                level1Symbols = letterSymbols;
            }
            else if (level1 == BulletType.RomanNumerals)
            {
                level1Symbols = romanNumeralSymbols;
            }
            if (level2 == BulletType.Numbers)
            {
                originalLevel2Symbols = numberSymbols;
            }
            else if (level2 == BulletType.Letters)
            {
                originalLevel2Symbols = letterSymbols;
            }
            else if (level2 == BulletType.RomanNumerals)
            {
                originalLevel2Symbols = romanNumeralSymbols;
            }
            var buffer = "";
            var title = "";
            var items = new List<string>();
            var level2Symbols = originalLevel2Symbols;
            foreach (var line in lines)
            {
                var parsedLine = this.ParseLine(line, delim);
                var lineKey = parsedLine[0];
                var lineValue = parsedLine[1];
                if (lineKey == Convert.ToString(level1Symbols[0]))
                {
                    level2Symbols = originalLevel2Symbols;
                    level1Symbols = level1Symbols.Slice(1, level1Symbols.Length).ToArray();
                    buffer = "";
                    buffer = buffer = line;
                    title = buffer;
                    items.Add(buffer);
                }
                else if (lineKey == Convert.ToString(level2Symbols[0]))
                {
                    level2Symbols = level2Symbols.Slice(1, level2Symbols.Length).ToArray();
                    buffer = "";
                    buffer = title + "\n" + buffer + line;
                    items.Add(buffer);
                }
                else
                {
                    buffer = buffer + line;
                    items[items.Count - 1] = buffer;
                }
            }
            return items;
        }

        private IList<string> ParseLine(string line, string delimiter)
        {
            var keyBuffer = "";
            var valueBuffer = "";
            var foundDelimiter = false;
            foreach (var character in line)
            {
                if (Convert.ToString(character) == delimiter)
                {
                    foundDelimiter = true;
                }
                else if (!foundDelimiter)
                {
                    keyBuffer = keyBuffer + character;
                }
                else
                {
                    valueBuffer = valueBuffer + character;
                }
            }
            return new string[] { keyBuffer, valueBuffer };
        }
    }

    public static class Extensions
    {
        public static IList<T> Slice<T>(this IList<T> source, int start, int end)
        {
            if (end < 0)
            {
                end = source.Count + end;
            }

            var count = end - start;
            var result = new T[count];
            for (int i = 0; i < count; ++i)
            {
                result[i] = source[start + i];
            }
            return result;
        }
    }
}
