package components;

//add imports
import cli.TraceLabSDK.*;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

@ComponentAttribute.Annotation(
        Name = "csvColumns",
        Description = "Read in column-separated data from a csv file",
        Author = "sarah",
        Version = "1.0",
        ConfigurationType = config.class)
@IOSpecAttribute.Annotation.__Multiple({
    @IOSpecAttribute.Annotation(IOType = IOSpecType.__Enum.Output, Name = "contents",
            DataType = String.class, Description = "Optional description for the tooltip")
})
public class csvColumns extends BaseComponent {

    private final config configFile;

    public csvColumns(ComponentLogger log) {
        super(log);
        configFile = new config();
        super.set_Configuration(configFile);
    }

    @Override
    public void Compute() {
        super.get_Logger().Trace("csvColumns worked!");

        //create 2 strings for the input and output files
        String in_fileName = configFile.getInputFile();
        String out_fileName = configFile.getOutputFile();

        //create a String to store the output of CsvWriter()
        String output = null;

        //store the output of CsvWriter() in output
        try {
            output = CsvWriter(in_fileName, out_fileName);
        } catch (IOException ex) {
            Logger.getLogger(csvColumns.class.getName()).log(Level.SEVERE, null, ex);
        }

        //store the output of CsvWriter() in the workspace
        super.get_Workspace().Store("contents", output);
    }

    //declare global variables
    static int count = 0;
    static String label1 = null;
    static String label2 = null;
    static double total1 = 0;
    static double total2 = 0;
    static double average1;
    static double average2;
    static double average3;

    public static String CsvWriter(String inputFile, String outputFile) throws FileNotFoundException, IOException {

        //declare local variables
        String message1;
        String message2;
        String message3;

        //collect data from csv file
        try (Scanner new_scanner = new Scanner(new File(inputFile))) {

            //gather labels and add numbers
            while (new_scanner.hasNext()) {
                count++;
                String next_Line = new_scanner.nextLine();
                String[] split = next_Line.split("\t");
                if (count == 1) {
                    label1 = split[0];
                    label2 = split[1];
                } else if (count > 1) {
                    total1 += Integer.parseInt(split[0]);
                    total2 += Integer.parseInt(split[1]);
                }
            } // END, while

            //calculate averages
            average1 = total1 / (count - 1);
            average2 = total2 / (count - 1);
            average3 = (average1 + average2) / 2;

            //format data
            message1 = "Average number of " + label1 + " per house: " + String.format("%.2f", average1);
            message2 = "Average number of " + label2 + " per house: " + String.format("%.2f", average2);
            message3 = "Average number of Pets per house: " + String.format("%.2f", average3);
        } // END, try (read in data from file)

        //create instance of FileWriter to be able to write data out to another file
        FileWriter fw = new FileWriter(outputFile);

        //write data out to file
        try (BufferedWriter bw = new BufferedWriter(fw)) {
            bw.append(message1);
            bw.append("\r\n");
            bw.append(message2);
            bw.append("\r\n");
            bw.append(message3);
        } // END, try (write data to file)
        //return messages to the workspace
        return (message1 + message2 + message3);
    } // END, CsvWriter()
} // END, class
