﻿using BulletsParser;
using HeadingSectionSplitter;
using SentenceParser;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using TraceLabSDK;
using TraceLabSDK.Types;

namespace CombinedPolicyParser
{
    [Component(
        Name = "Combined Policy Parser",
        Description = "Parse by heading/paragraph; parse by sentence; parse by bullets",
        Author = "Satrio Husodo",
        Version = "1.0",
        ConfigurationType = typeof(CombinedPolicyParserConfiguration)
    )]
    [IOSpec(IOSpecType.Output, "outputName", typeof(TLArtifactsCollection))]
    public class CombinedPolicyParserComponent : BaseComponent
    {
        public CombinedPolicyParserComponent(ComponentLogger log) : base(log) 
        {
            this.Configuration = new CombinedPolicyParserConfiguration();
        }

        public new CombinedPolicyParserConfiguration Configuration
        {
            get => base.Configuration as CombinedPolicyParserConfiguration;
            set => base.Configuration = value;
        }

        public override void Compute()
        {
            var i = 0;
            var collection = new TLArtifactsCollection();
            var inputDirectory = this.Configuration.Artifacts.Absolute;
            var outputDirectory = this.Configuration.OutputDirectory.Absolute;
            foreach (var item in this.GetItems(File.ReadLines(inputDirectory)))
            {
                i = i + 1;
                collection.Add(new TLArtifact(Convert.ToString(i), item));
                var outputFile = outputDirectory + @"\S" + i + ".txt";
                File.WriteAllText(outputFile, item);
            }
            base.Workspace.Store("outputName", collection);
        }

        private IEnumerable<string> GetItems(IEnumerable<string> lines)
        {
            switch (this.Configuration.ParseOptions)
            {
                case ParseOptions.HeadingAndSection:
                    return new HeadingSectionSplitterComponent(base.Logger).GetItems(lines);
                case ParseOptions.BulletedList:
                    var bulletType1 = this.Configuration.BulletTypeLevel1;
                    var bulletType2 = this.Configuration.BulletTypeLevel2;
                    var delimiter = this.Configuration.Delimiter;
                    return new BulletsParserComponent(base.Logger).GetItems(lines, bulletType1, bulletType2, delimiter);
                case ParseOptions.Sentence:
                    return new SentenceParserComponent(base.Logger).GetItems(lines);
                default:
                    throw new ComponentException("Invalid parse option given.");
            }
        }
    }
}
