﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Text.RegularExpressions;
using TraceLabSDK;
using TraceLabSDK.Types;

namespace DroolsRulesParser
{
    [Component(
        Name = "Drools Rules Parser",
        Description = "Parse a .drl into individual rule items",
        Author = "Satrio Husodo",
        Version = "1.0",
        ConfigurationType = typeof(DroolsRulesParserConfiguration)
    )]
    [IOSpec(IOSpecType.Output, "outputName", typeof(TLArtifactsCollection))]
    public class DroolsRulesParserComponent : BaseComponent
    {
        public DroolsRulesParserComponent(ComponentLogger log) : base(log)
        {
            this.Configuration = new DroolsRulesParserConfiguration();
        }

        public new DroolsRulesParserConfiguration Configuration
        {
            get => base.Configuration as DroolsRulesParserConfiguration;
            set => base.Configuration = value;
        }

        public override void Compute()
        {
            var i = 0;
            var collection = new TLArtifactsCollection();
            var inputDirectory = this.Configuration.Artifacts.Absolute;
            var outputDirectory = this.Configuration.OutputDirectory.Absolute;
            foreach (var item in this.GetItems(File.ReadLines(inputDirectory)))
            {
                i = i + 1;
                collection.Add(new TLArtifact(Convert.ToString(i), item));
                var outputFile = outputDirectory + @"\Rule" + i + ".txt";
                File.WriteAllText(outputFile, item);
            }
            base.Workspace.Store("outputName", collection);
        }

        public IEnumerable<string> GetItems(IEnumerable<string> lines)
        {
            var ruleNames = new List<string>();
            var parsedRules = new List<DroolsRule>();
            var ruleContent = new List<string>();
            var parsedRulesString = new List<string>();

            var numRules = 0;
            foreach (var line in lines)
            {
                var regex = new Regex("rule \"(.*)\"", RegexOptions.Compiled);
                foreach (Match match in regex.Matches(line))
                {
                    if (numRules > 0)
                    {
                        parsedRules[parsedRules.Count - 1].AddContent(ruleContent);
                    }
                    var matchedName = match.Groups[1].Value;
                    Console.WriteLine("match:" + matchedName);
                    ruleNames.Add(matchedName);
                    // Reset for new rule
                    parsedRules.Add(new DroolsRule(matchedName));
                    ruleContent = new List<string>();
                    numRules = numRules + 1;
                }
                ruleContent.Add(line);
            }
            // Ensure last item's content is added correctly
            parsedRules[parsedRules.Count - 1].AddContent(ruleContent);
            Console.WriteLine("\nRules found:");
            foreach (string r in ruleNames)
            {
                Console.WriteLine(r);
            }
            Console.WriteLine("\nRules objects:");
            int i = 1;
            foreach (DroolsRule r in parsedRules)
            {
                r.Display();
                i = i + 1;
                // Transfer rule object's content to add to list of string rules
                parsedRulesString.Add(r.parsedContentString);
            }
            Console.WriteLine("\nRules objects as strings:");
            return parsedRulesString;
        }
    }
}
