﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO.Abstractions;
using TraceLabSDK;
using TraceLabSDK.Types;

namespace HeadingSectionSplitter
{
    [Component(
        Name = "Heading Section Splitter",
        Description = "Split text into heading + content. Heading is a line within a specific length range. Content can be paragraph.",
        Author = "Satrio Husodo",
        Version = "1.0",
        ConfigurationType = typeof(HeadingSectionSplitterConfiguration)
    )]
    [IOSpec(IOSpecType.Output, "outputName", typeof(TLArtifactsCollection))]
    public class HeadingSectionSplitterComponent : BaseComponent
    {
        private IFileSystem fileSystem;

        public HeadingSectionSplitterComponent(ComponentLogger log) : base(log)
        {
            this.Configuration = new HeadingSectionSplitterConfiguration();
        }

        internal HeadingSectionSplitterComponent(ComponentLogger logger, IFileSystem fileSystem) : base(logger)
        {
            this.fileSystem = fileSystem;
            this.Configuration = new HeadingSectionSplitterConfiguration();
        }

        public new HeadingSectionSplitterConfiguration Configuration
        {
            get => base.Configuration as HeadingSectionSplitterConfiguration;
            set => base.Configuration = value;
        }

        public override void Compute()
        {
            var i = 0;
            var collection = new TLArtifactsCollection();
            var inputDirectory = this.Configuration.Artifacts.Absolute;
            var outputDirectory = this.Configuration.OutputDirectory.Absolute;
            foreach (var item in this.GetItems(this.fileSystem.File.ReadAllLines(inputDirectory)))
            {
                i = i + 1;
                collection.Add(new TLArtifact(Convert.ToString(i),item));
                var outputFile = outputDirectory + @"\S" + i + ".txt";
                this.fileSystem.File.WriteAllText(outputFile, item);
            }
            base.Workspace.Store("outputName", collection);
        }
        public IEnumerable<string> GetItems(IEnumerable<string> lines)
        {
            // Parse file's lines into list of strings
            // Strings will be returned and put into TL artifacts outside of this function

            // Lines that have length >4 and <25 will be deemed a heading and new item
            var MaxLimit = 25;
            var MinLimit = 4;

            var id = 0;
            var tempLine = "";
            var items = new List<string>();
            foreach (var line in lines)
            {
                //  Artifact collection is built up line by line
                if (line.Length < MaxLimit && line.Length > MinLimit)
                {
                    if (id > 0)
                    {
                        items.Add(tempLine);
                    }
                    tempLine = "";
                    tempLine = tempLine + line;
                    id = id + 1;
                }
                // Line is deemed as part of a paragraph
                else
                {
                    // Append paragraph in file already containing heading
                    tempLine = tempLine + "\n" + line;
                }
            }
            items.Add(tempLine);
            return items;
        }
    }
}
