package components;

import cli.TraceLabSDK.BaseComponent;
import cli.TraceLabSDK.ComponentAttribute;
import cli.TraceLabSDK.ComponentException;
import cli.TraceLabSDK.ComponentLogger;
import cli.TraceLabSDK.IOSpecAttribute;
import cli.TraceLabSDK.IOSpecType;
import cli.TraceLabSDK.Types.TLArtifact;
import cli.TraceLabSDK.Types.TLArtifactsCollection;
import cli.System.Collections.IEnumerator;
import cli.TraceLabSDK.TagAttribute;
import java.util.StringTokenizer;
import org.lemurproject.kstem.KrovetzStemmer;

/**
 *
 * @author David Farrar
 * This component was based upon the Java Porter Stemmer component developed by 
 * DePaul RE Lab
 */
@ComponentAttribute.Annotation(
        GuidIDString = "FE6D2C84-2219-1420-1814-7E884FD42085",
        Name = "KStem",
        Description = "Krovetz Stemmer. "
        + "Using KrovetzStemmer Version 3.4 from the Lemur Indri Project "
        + "See Below Licensing information from the Lemur Project.  \n"
        + "https://www.lemurproject.org/ \n"
        + "http://lemurproject.org/galago-license \n"
        + "Copyright 2003, Center for Intelligent Information Retrieval, "
        + "University of Massachusetts, Amherst. "
        + "All rights reserved. "
        + "\n"
        + "Redistribution and use in source and binary forms, with or without modification, "
        + "are permitted provided that the following conditions are met: "
        + "\n"
        + "1. Redistributions of source code must retain the above copyright notice, this "
        + "list of conditions and the following disclaimer. "
        + "\n"
        + "2. Redistributions in binary form must reproduce the above copyright notice, "
        + "this list of conditions and the following disclaimer in the documentation "
        + "and/or other materials provided with the distribution. "
        + "\n"
        + "3. The names \"Center for Intelligent Information Retrieval\" and "
        + "\"University of Massachusetts\" must not be used to endorse or promote products "
        + "derived from this software without prior written permission. To obtain "
        + "permission, contact info@ciir.cs.umass.edu. "
        + "\n"
        + "THIS SOFTWARE IS PROVIDED BY UNIVERSITY OF MASSACHUSETTS AND OTHER CONTRIBUTORS "
        + "\"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, "
        + "THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE "
        + "ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE "
        + "LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR"
        + "CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE "
        + "GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) "
        + "HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT "
        + "LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY "
        + "OUT OF THE USE  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF "
        + "SUCH DAMAGE. ",
        Author = "David Farrar",
        Version = "1.0")
@IOSpecAttribute.Annotation.__Multiple({
    @IOSpecAttribute.Annotation(IOType = IOSpecType.__Enum.Input, Name = "listOfArtifacts", DataType = cli.TraceLabSDK.Types.TLArtifactsCollection.class),
    @IOSpecAttribute.Annotation(IOType = IOSpecType.__Enum.Output, Name = "listOfArtifacts", DataType = cli.TraceLabSDK.Types.TLArtifactsCollection.class)
})
@TagAttribute.Annotation("Preprocessors.Stemmers")

public class kstem extends BaseComponent {
    
    public kstem(ComponentLogger log) {
        super(log);
    }

    /**
     *
     * @throws ComponentException
     */
    @Override
    public void Compute() throws ComponentException {
        ComponentLogger log = super.get_Logger();
        TLArtifactsCollection listOfArtifacts = (TLArtifactsCollection)super.get_Workspace().Load("listOfArtifacts");
        if(listOfArtifacts == null) {
            throw new ComponentException("Input listOfArtifacts is null");
        }
                
        KrovetzStemmer stemmer = new KrovetzStemmer();

        //Iterate through each artifact to stem the included text 
        IEnumerator en = listOfArtifacts.get_Keys().GetEnumerator();
        
        while (en.MoveNext()) {
            String tempOut = "";
            String artifactId = (String)en.get_Current();
            TLArtifact art = (TLArtifact)listOfArtifacts.get_Item(artifactId);
            
            //Krovetz Stemmer can only stem a single word at a time.  
            //Iterate through each word and compile stemmed output
            StringTokenizer tokenizer = new StringTokenizer(art.get_Text());
            while(tokenizer.hasMoreTokens()) {
                tempOut += " " + stemmer.stem(tokenizer.nextToken());
            }
            art.set_Text(tempOut);
        }

        super.get_Workspace().Store("listOfArtifacts", listOfArtifacts);
    }
}