package components;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.text.BreakIterator;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Set;
import java.util.regex.Pattern;
import org.apache.lucene.analysis.ASCIIFoldingFilter;
import org.apache.lucene.analysis.CharArraySet;
import org.apache.lucene.analysis.LowerCaseFilter;
import org.apache.lucene.analysis.PorterStemFilter;
import org.apache.lucene.analysis.StopFilter;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.standard.ClassicFilter;
import org.apache.lucene.analysis.standard.ClassicTokenizer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
import org.apache.lucene.util.Version;
import org.apache.solr.analysis.PatternReplaceFilter;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ArffSaver;
import cli.TraceLabSDK.*;
import java.util.logging.Level;
import java.util.logging.Logger;

@ComponentAttribute.Annotation(
        Name = "PaperClassifier",
        Description = "Component used to classifier a folder of pdf papers as either experimental or nonexperimental.",
        Author = "Clinton Woodson",
        Version = "1.0",
        ConfigurationType = Config.class)
@IOSpecAttribute.Annotation.__Multiple({
    @IOSpecAttribute.Annotation(IOType = IOSpecType.__Enum.Output, Name = "arff_out", DataType = String.class, Description = "Location of the arff file generated.")
})
public class PaperClassifier extends BaseComponent  {
    
    public PaperClassifier(ComponentLogger log) {
        super(log);
        config = new Config();
        super.set_Configuration(config);       
    }
    
    private final Config config;
    
    @Override
    public void Compute() {
        super.get_Logger().Trace("Starting");

        //your component implementation
        //String exampleInput = (String) super.get_Workspace().Load("example_input");
        //String exampleOutput = "mock value";
        //super.get_Workspace().Store("example_output", exampleOutput);
        
        FastVector atts;
	Instances data;
	double[] vals;
        
        // method and survey -> nonre, study and exp -> exp
        String expText = null, nonexpText = null;
        int numOfKeys = 10;
        //File expFolder = new File("C:\\Users\\Clint\\Desktop\\Hayes_Work\\Commitee work\\re2015\\exp_papers");
        //File nonexpFolder = new File("C:\\Users\\Clint\\Desktop\\Hayes_Work\\Commitee work\\re2015\\nonexp_papers");
        File expFolder = new File(config.getExperimentFolder().get_Absolute());
        File nonexpFolder = new File(config.getNonExperimentFolder().get_Absolute());
        File[] listOfExps = expFolder.listFiles();
        File[] listOfNonExps = nonexpFolder.listFiles();
//      List<Keyword> trainingKeys = new LinkedList<>();
//      super.get_Logger().Trace(config.getExperimentFolder().get_Absolute());
//      super.get_Logger().Trace(config.getNonExperimentFolder().get_Absolute());

        
        // 1. set up attributes
        atts = new FastVector();
        atts.addElement(new Attribute("file_name", (FastVector) null));
        atts.addElement(new Attribute("keywords", (FastVector) null));
        FastVector my_nominal_values = new FastVector(4); 
        my_nominal_values.addElement("exp");
        my_nominal_values.addElement("nonexp");
        atts.addElement(new Attribute("paper_type", my_nominal_values));
//        atts.addElement(new Attribute("paper_type", (FastVector) null));
        
        data = new Instances("MyResults", atts, 0);
        
        super.get_Logger().Trace("Variables created");
        
        for (File file : listOfExps) {
            if (file.isFile() && file.getName().endsWith(".pdf")) {
                System.out.println(file.getName());

                // Read in pdf and setup regex iterator
                PDFManager pdfManager = new PDFManager();
                pdfManager.setFilePath(expFolder + "\\" + file.getName());
                String text = "";
                try {
                    text = pdfManager.ToText();
                } catch (IOException ex) {
                    Logger.getLogger(PaperClassifier.class.getName()).log(Level.SEVERE, null, ex);
                }
                String lines[] = text.split("[\\r?\\n]+");
                String title = "";
                int t = 0;
                while ((lines[t].startsWith("Copy") || lines[t].startsWith(" ") || lines[t].startsWith("academic"))
                        && t < lines.length && lines.length > 1) {
                    t++;
                }
                if (lines.length > 1) {
                    title = lines[t] + lines[t + 1];
                }
                text = text.replaceAll("[\n\r]", " ");
                BreakIterator iterator = BreakIterator
                        .getSentenceInstance(Locale.US);
                iterator.setText(text);
                
                List<Keyword> keys = new LinkedList<Keyword>();
                try {
                    keys.addAll(guessFromString(text));
                } catch (IOException ex) {
                    Logger.getLogger(PaperClassifier.class.getName()).log(Level.SEVERE, null, ex);
                }
                expText = "";
                for (int i = 0; i < numOfKeys; i++) {
                    expText = expText + keys.get(i).getStem() + " ";
                    //System.out.println(keys.get(i).getStem() + " " + keys.get(i).getFrequency() + " " + keys.get(i).getTerms());
                }
                
                vals = new double[data.numAttributes()];
                // - numeric
                vals[0] = data.attribute(0).addStringValue(file.getName());
                vals[1] = data.attribute(1).addStringValue(expText);
                vals[2] = 0;
                
                data.add(new Instance(1.0, vals));
            }
        }
        
        super.get_Logger().Trace("Exp done");
        
        for (File file : listOfNonExps) {
            if (file.isFile() && file.getName().endsWith(".pdf")) {
                System.out.println(file.getName());

                // Read in pdf and setup regex iterator
                PDFManager pdfManager = new PDFManager();
                pdfManager.setFilePath(nonexpFolder + "\\" + file.getName());
                String text = "";
                try {
                    text = pdfManager.ToText();
                } catch (IOException ex) {
                    Logger.getLogger(PaperClassifier.class.getName()).log(Level.SEVERE, null, ex);
                }
                String lines[] = text.split("[\\r?\\n]+");
                String title = "";
                int t = 0;
                while ((lines[t].startsWith("Copy") || lines[t].startsWith(" ") || lines[t].startsWith("academic"))
                        && t < lines.length && lines.length > 1) {
                    t++;
                }
                if (lines.length > 1) {
                    title = lines[t] + lines[t + 1];
                }
                text = text.replaceAll("[\n\r]", " ");
                BreakIterator iterator = BreakIterator
                        .getSentenceInstance(Locale.US);
                iterator.setText(text);
                
                List<Keyword> keys = new LinkedList<Keyword>();
                try {
                    keys.addAll(guessFromString(text));
                } catch (IOException ex) {
                    Logger.getLogger(PaperClassifier.class.getName()).log(Level.SEVERE, null, ex);
                }
                nonexpText = "";
                for (int i = 0; i < numOfKeys; i++) {
                    nonexpText = nonexpText + keys.get(i).getStem() + " ";
                    //System.out.println(keys.get(i).getStem() + " " + keys.get(i).getFrequency() + " " + keys.get(i).getTerms());
                }
                
                vals = new double[data.numAttributes()];
                // - numeric
                vals[0] = data.attribute(0).addStringValue(file.getName());
                vals[1] = data.attribute(1).addStringValue(nonexpText);
                vals[2] = 1;
                
                data.add(new Instance(1.0, vals));
            }
        }
        
        super.get_Logger().Trace("Nonexp done");
        
        // 4. output data
        System.out.println(data);
        ArffSaver saver = new ArffSaver();
        saver.setInstances(data);
        try {
            saver.setFile(new File(config.getOutputFolder().get_Absolute() + "\\exp_classifier.arff"));
        } catch (IOException ex) {
            Logger.getLogger(PaperClassifier.class.getName()).log(Level.SEVERE, null, ex);
        }
        try {
            saver.writeBatch();
        } catch (IOException ex) {
            Logger.getLogger(PaperClassifier.class.getName()).log(Level.SEVERE, null, ex);
        }
        super.get_Workspace().Store("arff_out", config.getNonExperimentFolder().get_Absolute() + "\\exp_classifier.arff");
        super.get_Logger().Trace("Arff file created");
        //System.out.println("Arff File created.");
    }
    
    public static String stem(String term) throws IOException {

        TokenStream tokenStream = null;
        try {

            // tokenize
            tokenStream = new ClassicTokenizer(Version.LUCENE_36, new StringReader(term));
            // stem
            tokenStream = new PorterStemFilter(tokenStream);

            // add each token in a set, so that duplicates are removed
            Set<String> stems = new HashSet<String>();
            CharTermAttribute token = tokenStream.getAttribute(CharTermAttribute.class);
            while (tokenStream.incrementToken()) {
                stems.add(token.toString());
            }

            // if no stem or 2+ stems have been found, return null
            if (stems.size() != 1) {
                return null;
            }
            String stem = stems.iterator().next();
            // if the stem has non-alphanumerical chars, return null
            if (!stem.matches("[a-zA-Z0-9-]+")) {
                return null;
            }
            if (stem.length() == 1) {
                return null;
            }

            return stem;

        } finally {
            if (tokenStream != null) {
                tokenStream.close();
            }
        }

    }
    
    public static <T> T find(Collection<T> collection, T example) {
        for (T element : collection) {
            if (element.equals(example)) {
                return element;
            }
        }
        collection.add(example);
        return example;
    }

    public static List<Keyword> guessFromString(String input) throws IOException {
        
        String[] STOP_WORDS ={"a", "about", "above", "across", "after", "afterwards", "again", 
		"against", "all", "almost", "alone", "along", "already", "also", "although", "always", "am", "among", "amongst", 
		"amoungst", "amount", "an", "and", "another", "any", "anyhow", "anyone", "anything", "anyway", "anywhere", "are", 
		"around", "as", "at", "back", "be", "became", "because", "become", "becomes", "becoming", "been", "before", 
		"beforehand", "behind", "being", "below", "beside", "besides", "between", "beyond", "bill", "both", "bottom", 
		"but", "by", "call", "can", "cannot", "cant", "co", "computer", "con", "could", "couldnt", "cry", "de", "describe", 
		"detail", "do", "done", "down", "due", "during", "each", "eg", "eight", "either", "eleven", "else", "elsewhere", 
		"empty", "enough", "etc", "even", "ever", "every", "everyone", "everything", "everywhere", "except", "few", "fifteen", 
		"fify", "fill", "find", "fire", "first", "five", "for", "former", "formerly", "forty", "found", "four", "from", "front", 
		"full", "further", "get", "give", "go", "had", "has", "hasnt", "have", "he", "hence", "her", "here", "hereafter", "hereby", 
		"herein", "hereupon", "hers", "herse", "him", "himse", "his", "how", "however", "hundred", "i", "ie", "if", "in", "inc", 
		"indeed", "interest", "into", "is", "it", "its", "itse", "keep", "last", "latter", "latterly", "least", "less", "ltd", 
		"made", "many", "may", "me", "meanwhile", "might", "mill", "mine", "more", "moreover", "most", "mostly", "move", "much", 
		"must", "my", "myse", "name", "namely", "neither", "never", "nevertheless", "next", "nine", "no", "nobody", "none", 
		"noone", "nor", "not", "nothing", "now", "nowhere", "of", "off", "often", "on", "once", "one", "only", "onto", "or", 
		"other", "others", "otherwise", "our", "ours", "ourselves", "out", "over", "own", "part", "per", "perhaps", "please", 
		"put", "rather", "re", "same", "see", "seem", "seemed", "seeming", "seems", "serious", "several", "she", "should", 
		"show", "side", "since", "sincere", "six", "sixty", "so", "some", "somehow", "someone", "something", "sometime", 
		"sometimes", "somewhere", "still", "such", "system", "take", "ten", "than", "that", "the", "their", "them", "themselves", 
		"then", "thence", "there", "thereafter", "thereby", "therefore", "therein", "thereupon", "these", "they", "thick", "thin", 
		"third", "this", "those", "though", "three", "through", "throughout", "thru", "thus", "to", "together", "too", "top", 
		"toward", "towards", "twelve", "twenty", "two", "un", "under", "until", "up", "upon", "us", "use", "uses", "used", "using", 
                "very", "via", "was", "we", "well", "were", "what", "whatever", "when", "whence", "whenever", "where", "whereafter", "whereas",
                "whereby", "wherein", "whereupon", "wherever", "whether", "which", "while", "whither", "who", "whoever", "whole", "whom", "whose",
                "why", "will", "with", "within", "without", "would", "yet", "you", "your", "yours", "yourself", "yourselves"};

        TokenStream tokenStream = null;
        try {

            // hack to keep dashed words (e.g. "non-specific" rather than "non" and "specific")
            input = input.replaceAll("-+", "-0");
            // replace any punctuation char but apostrophes and dashes by a space
            input = input.replaceAll("[\\p{Punct}&&[^'-]]+", " ");
            // replace most common english contractions
            input = input.replaceAll("(?:'(?:[tdsm]|[vr]e|ll))+\\b", "");

            // tokenize input
            tokenStream = new ClassicTokenizer(Version.LUCENE_36, new StringReader(input));
            // to lowercase
            tokenStream = new LowerCaseFilter(Version.LUCENE_36, tokenStream);
            // remove dots from acronyms (and "'s" but already done manually above)
            tokenStream = new ClassicFilter(tokenStream);
            // convert any char to ASCII
            tokenStream = new ASCIIFoldingFilter(tokenStream);
            // remove english stop words
            CharArraySet stopSet = CharArraySet.copy(Version.LUCENE_36, StandardAnalyzer.STOP_WORDS_SET);
            stopSet.addAll(Arrays.asList(STOP_WORDS));
            tokenStream = new StopFilter(Version.LUCENE_36, tokenStream, stopSet);
            //tokenStream = new StopFilter(Version.LUCENE_36, tokenStream, StandardAnalyzer.STOP_WORDS_SET);//EnglishAnalyzer.getDefaultStopSet());
            // remove numbers
            Pattern pattern = Pattern.compile("(\\d)");
            tokenStream = new PatternReplaceFilter(tokenStream, pattern, "", true);

            List<Keyword> keywords = new LinkedList<Keyword>();
            CharTermAttribute token = tokenStream.getAttribute(CharTermAttribute.class);
            while (tokenStream.incrementToken()) {
                String term = token.toString();
                // stem each term
                String stem = stem(term);
                if (stem != null) {
                    // create the keyword or get the existing one if any
                    Keyword keyword = find(keywords, new Keyword(stem.replaceAll("-0", "-")));
                    // add its corresponding initial token
                    keyword.add(term.replaceAll("-0", "-"));
                }
            }

            // reverse sort by frequency
            Collections.sort(keywords);

            return keywords;

        } finally {
            if (tokenStream != null) {
                tokenStream.close();
            }
        }

    }
}
