package components;

import cli.TraceLabSDK.*;
import cli.TraceLabSDK.Types.Generics.Collections.StringList;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

@ComponentAttribute.Annotation(
        Name = "Parse Policy",
        Description = "Parses policy.json file",
        Author = "Gabbi",
        Version = "1.0",
        ConfigurationType=Config.class)

@IOSpecAttribute.Annotation.__Multiple({
        
})

@TagAttribute.Annotation("Parser")

public class ParsePolicy extends BaseComponent {
    
    private final Config config;
    
    public ParsePolicy(ComponentLogger log) {
        super(log);
        config = new Config();
        super.set_Configuration(config);
    }
    
    @Override
    public void Compute() {
        
        try{
            String inputFile = config.getInputFile();
        
            JSONParser parser = new JSONParser();
            Object obj = parser.parse(new FileReader(inputFile));
            JSONObject policytable = (JSONObject) obj;
            JSONArray policy = (JSONArray) ((JSONObject) policytable.get("policy-table")).get("policy");
            
            //Fill data from the policy array
            for(int i=0;i<policy.size();i++){
                JSONObject policyobject = (JSONObject) policy.get(i);
                Long zoneId = (Long) policyobject.get("zoneId");
                Boolean defaultAction = (Boolean) policyobject.get("defaultAction");
                
                super.get_Logger().Info("Policy: " + (i+1));
                super.get_Logger().Info("   ZoneId: " + zoneId);
                super.get_Logger().Info("   DefaultAction: " + defaultAction);
                    
                JSONArray zones = (JSONArray) policyobject.get("zones");
                
                if(zones!=null){
                    for(int j=0;j<zones.size();j++){
                        JSONObject zonesobject = (JSONObject) zones.get(j);
                        JSONArray exceptions = (JSONArray) zonesobject.get("exceptions");
                        
                        super.get_Logger().Info("   Zones: ");
                        super.get_Logger().Info("       Exceptions: ");
                        
                        if(exceptions!=null){
                            for(int k=0;k<exceptions.size();k++){
                                JSONObject exceptionsobject = (JSONObject) exceptions.get(k);
                                String srcUser = (String) exceptionsobject.get("srcUser");
                                Boolean isAllowed = (Boolean) exceptionsobject.get("isAllowed");
                                
                                super.get_Logger().Info("       srcUser: " + srcUser);
                                super.get_Logger().Info("       isAllowed: " + isAllowed);
                            }
                        }
             
                        Long dstZoneId = (Long) zonesobject.get("dstzoneId");
                        Boolean isAllowed2 = (Boolean) zonesobject.get("isAllowed");
                               
                        super.get_Logger().Info("   DstZoneId: "+ dstZoneId);
                        super.get_Logger().Info("   IsAllowed: " + isAllowed2);
                        }
                    }
                    
          
            }
        }
        
        catch(FileNotFoundException e){
            super.get_Logger().Info(e);
        }
        
        catch(IOException e){
            super.get_Logger().Info(e);
        }
        
        catch(ParseException e){
            super.get_Logger().Info(e);
        }
    }
}
