﻿using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using TraceLabSDK;
using TraceLabSDK.Types;

namespace SimilarityMatrixFilter.Tests
{
    [TestClass]
    public class SimilarityMatrixFilterComponentTests
    {
        private IDictionary<string, object> workspace;

        private SimilarityMatrixFilterComponent component;

        [TestInitialize]
        public void TestInitialize()
        {
            this.workspace = new Dictionary<string, object>();

            var logger = new Mock<ComponentLogger>();
            this.component = new SimilarityMatrixFilterComponent(logger.Object);

            var workspace = new Mock<IWorkspace>();
            workspace.Setup(mock => mock.Load(It.IsAny<string>())).Returns<string>(str => this.workspace[str]);
            workspace.Setup(mock => mock.Store(It.IsAny<string>(), It.IsAny<object>())).Callback<string, object>((str, obj) => this.workspace[str] = obj);
            this.component.Workspace = workspace.Object;
        }

        [TestMethod]
        public void TestConfigurationIsCustomType()
        {
            Assert.IsInstanceOfType(this.component.Configuration, typeof(SimilarityMatrixFilterConfiguration));
        }

        [TestMethod]
        public void TestComputeFiltersSimilarityMatrix()
        {
            var matrix = new TLSimilarityMatrix();
            matrix.AddLink("A", "B", 0.05);
            matrix.AddLink("C", "D", 0.10);
            matrix.AddLink("E", "F", 0.15);
            this.component.Workspace.Store("SimilarityMatrix", matrix);

            this.component.Compute();
            var result = this.component.Workspace.Load("FilteredMatrix") as TLSimilarityMatrix;

            var expected = new TLSimilarityMatrix();
            expected.AddLink("C", "D", 0.10);
            expected.AddLink("E", "F", 0.15);
            CollectionAssert.AreEquivalent(expected.AllLinks, result.AllLinks);
        }
    }
}
