﻿using System.ComponentModel;
using System.Linq;
using TraceLabSDK;
using TraceLabSDK.Types;

namespace SimilarityMatrixFilter
{
    [Component(
        Name = "Similarity Matrix Filter",
        Description = "Filters a similarity matrix given a relevance score threshold.",
        Author = "Jared Payne",
        Version = "1.0",
        ConfigurationType = typeof(SimilarityMatrixFilterConfiguration)
    )]
    [IOSpec(IOType = IOSpecType.Input, Name = "SimilarityMatrix", DataType = typeof(TLSimilarityMatrix))]
    [IOSpec(IOType = IOSpecType.Output, Name = "FilteredMatrix", DataType = typeof(TLSimilarityMatrix))]
    [Tag("Filters")]
    [Tag("Postprocessors")]
    public class SimilarityMatrixFilterComponent : BaseComponent
    {
        public SimilarityMatrixFilterComponent(ComponentLogger logger) : base(logger)
        {
            this.Configuration = new SimilarityMatrixFilterConfiguration();
        }

        public new SimilarityMatrixFilterConfiguration Configuration
        {
            get => base.Configuration as SimilarityMatrixFilterConfiguration;
            set => base.Configuration = value;
        }

        public override void Compute()
        {
            var matrix = new TLSimilarityMatrix();
            (Workspace.Load("SimilarityMatrix") as TLSimilarityMatrix).AllLinks
                .Where(link => link.Score >= this.Configuration.Threshold).ToList()
                .ForEach(link => matrix.AddLink(link.SourceArtifactId, link.TargetArtifactId, link.Score));
            Workspace.Store("FilteredMatrix", matrix);
        }
    }
}
