﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.IO;
using System.IO.Abstractions.TestingHelpers;
using TraceLabSDK;
using TraceLabSDK.Component.Config;
using TraceLabSDK.Types;

namespace SimilarityMatrixVotingMerge.Tests
{
    [TestClass]
    public class SimilarityMatrixVotingMergeComponentTests
    {
        private MockFileSystem fileSystem;

        private string workingDirectory;

        private SimilarityMatrixVotingMergeComponent component;

        private IDictionary<string, object> workspace;

        [TestInitialize]
        public void TestInitialize()
        {
            this.fileSystem = new MockFileSystem();
            this.workingDirectory = this.fileSystem.Directory.GetCurrentDirectory();

            var logger = new Mock<ComponentLogger>();
            this.component = new SimilarityMatrixVotingMergeComponent(logger.Object, this.fileSystem);

            var workspace = new Mock<IWorkspace>();
            this.workspace = new Dictionary<string, object>();
            workspace.Setup(mock => mock.Load(It.IsAny<string>()))
                .Returns<string>(str => this.workspace[str]);
            workspace.Setup(mock => mock.Store(It.IsAny<string>(), It.IsAny<object>()))
                .Callback<string, object>((str, obj) => this.workspace[str] = obj);
            this.component.Workspace = workspace.Object;
        }

        [TestMethod]
        public void TestConfigurationIsCustomType()
        {
            Assert.IsInstanceOfType(this.component.Configuration, typeof(SimilarityMatrixVotingMergeConfiguration));
        }

        [TestMethod]
        public void TestComputeMergesSimilarityMatrices()
        {
            var matricesDirectory = this.fileSystem.Path.Combine(this.workingDirectory, "Matrices");
            this.fileSystem.AddDirectory(matricesDirectory);
            var matrix1 = new MockFileData("A\tB\t0.10\nC\tD\t0.20");
            this.fileSystem.AddFile(this.fileSystem.Path.Combine(matricesDirectory, "Matrix1.txt"), matrix1);
            var matrix2 = new MockFileData("C\tD\t0.30\nE\tF\t0.40");
            this.fileSystem.AddFile(this.fileSystem.Path.Combine(matricesDirectory, "Matrix2.txt"), matrix2);
            var matrix3 = new MockFileData("A\tB\t0.50\nE\tF\t0.60");
            this.fileSystem.AddFile(this.fileSystem.Path.Combine(matricesDirectory, "Matrix3.txt"), matrix3);

            var directoryPath = new DirectoryPath();
            directoryPath.Init(matricesDirectory);
            this.component.Configuration.DirectoryPath = directoryPath;

            this.component.Compute();

            var expected = new TLSimilarityMatrix();
            expected.AddLink("A", "B", 0.30);
            expected.AddLink("C", "D", 0.25);
            expected.AddLink("E", "F", 0.50);
            var result = this.component.Workspace.Load("MergedMatrix") as TLSimilarityMatrix;
            CollectionAssert.AreEquivalent(expected.AllLinks, result.AllLinks);
        }
    }
}
