﻿using System;
using System.IO.Abstractions.TestingHelpers;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TraceLabSDK.Component.Config;

namespace SimilarityMatrixVotingMerge.Tests
{
    [TestClass]
    public class SimilarityMatrixVotingMergeConfigurationTests
    {
        private SimilarityMatrixVotingMergeConfiguration configuration;

        [TestInitialize]
        public void TestInitialize()
        {
            this.configuration = new SimilarityMatrixVotingMergeConfiguration(new MockFileSystem());
        }

        [TestMethod]
        public void TestThrowsArgumentNullExceptionWhenDirectoryPathIsNull()
        {
            Assert.ThrowsException<ArgumentNullException>(() => this.configuration.DirectoryPath = null);
        }

        [TestMethod]
        public void TestThrowsArgumentExceptionWhenDirectoryPathDoesNotExist()
        {
            var path = new DirectoryPath();
            path.Init(@"C:\NotADirectory");

            Assert.ThrowsException<ArgumentException>(() => this.configuration.DirectoryPath = path);
        }

        [TestMethod]
        public void TestThresholdDefaultValueIsFiftyPercent()
        {
            Assert.AreEqual(0.5, this.configuration.Threshold);
        }

        [TestMethod]
        public void TestThrowsArgumentExceptionWhenThresholdIsLessThanZero()
        {
            Assert.ThrowsException<ArgumentException>(() => this.configuration.Threshold = -0.1);
        }

        [TestMethod]
        public void TestThrowsArgumentExceptionWhenThresholdIsGreaterThanOne()
        {
            Assert.ThrowsException<ArgumentException>(() => this.configuration.Threshold = 1.1);
        }
    }
}
