﻿using Common;
using System;
using System.ComponentModel;
using System.IO.Abstractions;
using TraceLabSDK.Component.Config;

namespace SimilarityMatrixVotingMerge
{
    public class SimilarityMatrixVotingMergeConfiguration : FileSystemConfiguration
    {
        private DirectoryPath directoryPath;

        private double threshold = 0.5;

        public SimilarityMatrixVotingMergeConfiguration() : this(new FileSystem()) { }

        internal SimilarityMatrixVotingMergeConfiguration(IFileSystem fileSystem) : base(fileSystem) { }

        [DisplayName("Input directory")]
        [Description("The directory that contains the similarity matrices.")]
        public DirectoryPath DirectoryPath
        {
            get => this.directoryPath;
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Directory path cannot be null.");
                }
                if (!this.FileSystem.Directory.Exists(value.Absolute))
                {
                    throw new ArgumentException("Directory path is not a directory.");
                }
                this.directoryPath = value;
            }
        }

        [DefaultValue(0.5)]
        [DisplayName("Threshold")]
        [Description("Filter out links that are not in at least this percent of the input matrices.")]
        public double Threshold
        {
            get => this.threshold;
            set
            {
                if (value < 0.0 || value > 1.0)
                {
                    throw new ArgumentException("Threshold must be between 0.0 and 1.0.");
                }
                this.threshold = value;
            }
        }
    }
}
