package components;

import cli.TraceLabSDK.*;
import cli.TraceLabSDK.Types.Generics.Collections.StringList;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.io.PrintWriter;

@ComponentAttribute.Annotation(
        Name = "Sum Rows",
        Description = "Sums the rows of integers from a .txt file and saves them to a new .txt file",
        Author = "Gabbi",
        Version = "1.0",
        ConfigurationType=Config.class)

@IOSpecAttribute.Annotation.__Multiple({
    @IOSpecAttribute.Annotation(IOType = IOSpecType.__Enum.Output, Name = "Contents",
            DataType = StringList.class)   
        
})

@TagAttribute.Annotation("File Reader")

public class SumRows extends BaseComponent {
    
    
    private final Config config;
    
    public SumRows(ComponentLogger log) {
        super(log);
        config = new Config();
        super.set_Configuration(config);
    }
    
    @Override
    public void Compute() {
        
        String line;
        int sum;
        String value;
        
        //Create a StringList to store the output 
        StringList sumdata = new StringList();
                
        //Create a string for the input and output files
        String input_file = config.getInputFile();  
        String output_file = config.getOutputFile(); 
 
        try{
            
           //Open the input file
           Scanner scanner = new Scanner(new File(input_file));
    
           //Create an output file to store the values
           PrintWriter pw = new PrintWriter(output_file);
 
           //Scan through the input file line by line
           while(scanner.hasNextLine()){
               sum =0;
               line = scanner.nextLine();
               Scanner row = new Scanner(line);
               
               //Add integers to sum
               while(row.hasNextInt()){
                   sum+=row.nextInt();
                   value = String.valueOf(sum); //Convert sum to a string
                   sumdata.Add(value);  //Add sum string to sumdata StringList
               }
               
               //Print sum to output file
               pw.println(sum);
           }
           
           //Print values to workspace
           super.get_Workspace().Store("Contents", sumdata);
           
           //Close the output file
           pw.close();
        }
        
        catch(FileNotFoundException exception){
            super.get_Logger().Info(exception);
        }
     
        
      
}}
