package components;
import cli.TraceLabSDK.BaseComponent;
import cli.TraceLabSDK.ComponentAttribute;
import cli.TraceLabSDK.ComponentException;
import cli.TraceLabSDK.ComponentLogger;
import cli.TraceLabSDK.IOSpecAttribute;
import cli.TraceLabSDK.IOSpecType;
import cli.TraceLabSDK.TagAttribute;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
//import org.jdom2.Document;
//import org.jdom2.input.SAXBuilder;
//import org.jdom2.JDOMException;
import java.util.ArrayList;
import java.util.List;
import org.jdom2.Attribute;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;



@ComponentAttribute.Annotation(
        Name = "ULOC",
        Description = "Common Arguments:\n"
        + "--by-file Report results for every source file encountered.\n"
        + "--by-file-by-lang Report results for every source file encountered\n"
        + "in addition to reporting by language."
        + "--csv Write the results as comma separated values.\n"
        + "--report-file=<file> Write the results to <file> instead of STDOUT.\n"
        + "--quiet Suppress all information messages except for\n the final\n "
        + "report.\n"
        + ""
        + "This component uses the CLOC utility by Al Danial al.danial@gmail.com\n"
        + "Website: cloc.sourceforge.net\n"
        + "Al Danial Copyright (c) 2006-2015\n"
        + "CLOC License: This program is free software;"
        + " you can redistribute it and/or modify it under"
        + " the terms of the GNU General Public License"
        + " as published by the Free Software Foundation;"
        + " either version 2 of the License, or (at your option)"
        + " any later version.",
        Author = "David Farrar",
        Version = "1.0",
        ConfigurationType=Config.class)
@IOSpecAttribute.Annotation.__Multiple({
    @IOSpecAttribute.Annotation(IOType = IOSpecType.__Enum.Input, Name = "InputFile",
        DataType = String.class),
    @IOSpecAttribute.Annotation(IOType = IOSpecType.__Enum.Output, Name = "uLoC",
        DataType = String.class)
})

@TagAttribute.Annotation("ULOC")
public class ULOC extends BaseComponent {

    private final Config config;
       //Example call of cloc.exe for testing purposes
//    "c:\\cloc.exe --csv --by-file C:\\cygwin64\\home\\David\\research\\make_1.4\\make\\versions.alt\\versions.orig\\v0";

    public ULOC(ComponentLogger log) {
         super(log);
         config = new Config();
         super.set_Configuration(config);
    }

    @Override
    public void Compute() throws ComponentException {

        ComponentLogger log = super.get_Logger();
        log.Info("Initialized");

        String xmlOutput = null;
         try {
            log.Info("Attempt callCommandLine");
            //Call the CLOC.exe utiltity to generate the XML analysis of the selected file.
            xmlOutput = callCommandLine();
            log.Info("Called callCommandLine");
        } catch (InterruptedException ex) {
            log.Info(ex.toString());
        }
        catch (IOException ex) {
            log.Info(ex.toString());
        }
            
        catch (JDOMException ex) {
            log.Info(ex.toString());
        }
        
        super.get_Workspace().Store("uLoC", xmlOutput);
    }

    /**
     *
     * @return
     * @throws InterruptedException
     * @throws IOException
     * @throws JDOMException
     */
    public String callCommandLine() throws InterruptedException, IOException, JDOMException{
        String outputString = "";
        String CMD = null;

        //Calling the cloc.exe program.  This needs to be set to an input
        String clocExecutable = "C:\\cloc.exe";  
        
        //Commandline arguments for cloc.exe 
        //Should make selectable by user.  
        String clocArguments = "--xml --by-file --quiet";
        //Get the selected file name and path form configuration input.
        String inspectionFile = config.getInspectionFile().get_Absolute();

try {
            //Build call for external program
            CMD = clocExecutable + " " + clocArguments + " " + inspectionFile;

            //Run the external program 
            Process process = Runtime.getRuntime().exec(CMD);

            // Get input streams
            BufferedReader stdInput = new BufferedReader(new InputStreamReader(process.getInputStream()));
            BufferedReader stdError = new BufferedReader(new InputStreamReader(process.getErrorStream()));

            // Read command standard output
            String s;
//            System.out.println("Standard output: ");
            while ((s = stdInput.readLine()) != null) {
                outputString += s + "\n";
                System.out.println(s);
            }

            // Read command errors
            System.out.println("Standard error: ");
            while ((s = stdError.readLine()) != null) {
                outputString += s + "\n";
                System.out.println(s);
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }

        //Inplementation for creating a List of strings containing each file entry
        //Disabled due to issue wiht using JDOM2 in TraceLAB
//        List<String> fileList = parseXML(outputString);
    
        return outputString;
    }

    /**
     * Method currently not being called due to issue with using JDOM2 in TraceLab
     * Issue with JDOM2 may be due to not correctly passing dependent Jars to 
     * IKVM call when building DLL.
     * @param xmlInput
     * @return
     * @throws IOException
     * @throws JDOMException
     */
    public  List<String> parseXML(String xmlInput) throws IOException, JDOMException {
        ComponentLogger log = super.get_Logger();
        log.Info("Initialized");
        
        //List of strings to contain XML data for each file
        List<String> fileList;
        fileList = new ArrayList<String>();

        //Convert input string containing XML to XML object for parsing.
        SAXBuilder jdomBuilder = new SAXBuilder();
        ByteArrayInputStream input = new ByteArrayInputStream(xmlInput.getBytes("UTF-8"));
        org.jdom2.Document jdomDocument = jdomBuilder.build(input);

        //Select the Root element of the XML data.
        System.out.println("Root Element Name " +jdomDocument.getRootElement().getName());
        org.jdom2.Element rootElement = jdomDocument.getRootElement();

        //Split the XML Data based on the files tag
        Element files = rootElement.getChild("files");
        List<Element> fileChildren = files.getChildren();
        for (int i = 0; i < fileChildren.size(); i++) {
            Element fileChild = fileChildren.get(i);
            List<Attribute> childAttributes = fileChild.getAttributes();
            //Collect the data associated with each file and add it to the current file string.
            for (Attribute childAttribute : childAttributes) {
                System.out.println("Child Attributes: " + childAttribute.getName());
                Attribute attribute = fileChild.getAttribute("name");
                if(attribute != null) {
                    fileList.add(attribute.getValue());
                } else {
                    break;
                }
                attribute = fileChild.getAttribute("code");
                if(attribute != null) {
                    fileList.add(attribute.getValue());
                } else {
                    break;
                }
            }
        }
    fileList.add("temp");
    return fileList;
    }
}
